package jp.asciimw.androidbook.chapter2.andlogo.canvas;

import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.PointF;
import android.graphics.Rect;
import android.graphics.RectF;
import android.graphics.drawable.Drawable;

public class Turtle {
    private int width;
    private int height;
    private Drawable drawable;
    private RectF rect;
    private float direction;
    private boolean visible;
    private boolean penUp;
    private Paint paint;

    public Turtle(Drawable drawable, Rect viewRect) {
        this.drawable = drawable;
        this.width = drawable.getIntrinsicWidth();
        this.height = drawable.getIntrinsicHeight();
        init(drawable, viewRect);
    }

    public void init(Drawable drawable, Rect viewRect) {
        this.rect = new RectF(0.0f, 0.0f, width, height);
        rect.offset(viewRect.right / 2 - width / 2, viewRect.bottom / 2 - height / 2);
        direction = 90.0f;
        visible = true;
        penUp = false;
        paint = new Paint(Paint.ANTI_ALIAS_FLAG);
        paint.setColor(Color.RED);
        paint.setStrokeWidth(1);
        paint.setStyle(Paint.Style.STROKE);
    }

    public void draw(Canvas canvas) {
        if (visible) {
            canvas.save();
            canvas.rotate(direction * -1.0f, getX(), getY());
            drawable.setBounds((int) rect.left, (int) rect.top, (int) rect.right, (int) rect.bottom);
            drawable.draw(canvas);
            canvas.restore();
        }
    }

    public PointF getCurrentPoint() {
        return new PointF(rect.centerX(), rect.centerY());
    }

    public Paint getCurrentPaint() {
        return paint;
    }

    public void setPenColor(int color) {
        Paint tmp = new Paint(Paint.ANTI_ALIAS_FLAG);
        tmp.setColor(color);
        tmp.setStrokeWidth(paint.getStrokeWidth());
        tmp.setStyle(Paint.Style.STROKE);
        paint = tmp;
    }

    public void setPenStrokeWidth(int width) {
        Paint tmp = new Paint(Paint.ANTI_ALIAS_FLAG);
        tmp.setColor(paint.getColor());
        tmp.setStrokeWidth(width);
        tmp.setStyle(Paint.Style.STROKE);
        paint = tmp;
    }

    public float getX() {
        return rect.centerX();
    }

    public float getY() {
        return rect.centerY();
    }

    public void move(float dx, float dy) {
        rect.offset(dx, dy);
    }

    public void right(float angle) {
        left(-angle);
    }

    public void left(float angle) {
        direction += angle;
        while (direction > 360.0f) {
            direction -= 360.0f;
        }
        while (direction < 0.0f) {
            direction += 360.0f;
        }
    }

    public float getDirection() {
        return direction;
    }

    public void hide() {
        visible = false;
    }

    public void show() {
        visible = true;
    }

    public boolean isHidden() {
        return !visible;
    }

    public void penUp() {
        penUp = true;
    }

    public void penDown() {
        penUp = false;
    }

    public boolean isPenUp() {
        return penUp;
    }
}
