package jp.asciimw.androidbook.chapter2.andlogo.core;

import android.content.Context;
import android.graphics.Color;

import java.util.HashMap;
import java.util.Map;

import jp.asciimw.androidbook.chapter2.andlogo.core.LogoToken.LogoTokenKind;

public class LogoCommandInvoker {
    private LogoToken token;
    private Map<String, String> dictionary = new HashMap<String, String>();
    private static Map<String, Integer> colorTable = new HashMap<String, Integer>();
    public LogoCommandInvoker(LogoCommandReceiver receiver) {
        this.receiver = receiver;
    }
    static {
        colorTable.put("blue", Color.BLUE);
        colorTable.put("cyan", Color.CYAN);
        colorTable.put("darkgray", Color.DKGRAY);
        colorTable.put("gray", Color.GRAY);
        colorTable.put("green", Color.GREEN);
        colorTable.put("lightgray", Color.LTGRAY);
        colorTable.put("magenta", Color.MAGENTA);
        colorTable.put("red", Color.RED);
        colorTable.put("white", Color.WHITE);
        colorTable.put("yellow", Color.YELLOW);
    }
    private LogoCommandParser parser = new LogoCommandParser();
    private LogoCommandReceiver receiver;
    public void invoke(String command) {
        parser.setSourceString(command);
        invokeCommandImpl(0);
    }
    void invokeCommandImpl(int level) {
        String commandName;
        String code;
        token = parser.getToken();
        while (token.kind != LogoTokenKind.KIND_EOI
                && token.kind != LogoTokenKind.KIND_INVALID_TOKEN) {
            switch (token.kind) {
            case KIND_FORWARD:
                token = parser.getToken();
                if (token.kind == LogoTokenKind.KIND_FLOAT
                        || token.kind == LogoTokenKind.KIND_INTEGER) {
                    receiver.forward(token.value);
                    token = parser.getToken();
                } else {
                    notifyError("forwardには距離を指定して下さい");
                    return;
                }
                break;

            case KIND_BACK:
                token = parser.getToken();
                if (token.kind == LogoTokenKind.KIND_FLOAT
                        || token.kind == LogoTokenKind.KIND_INTEGER) {
                    receiver.back(token.value);
                    token = parser.getToken();
                } else {
                    notifyError("backには距離を指定して下さい");
                    return;
                }
                break;

            case KIND_LEFT:
                token = parser.getToken();
                if (token.kind == LogoTokenKind.KIND_FLOAT
                        || token.kind == LogoTokenKind.KIND_INTEGER) {
                    receiver.left(token.value);
                    token = parser.getToken();
                } else {
                    notifyError("leftには角度を指定して下さい");
                    return;
                }
                break;

            case KIND_RIGHT:
                token = parser.getToken();
                if (token.kind == LogoTokenKind.KIND_FLOAT
                        || token.kind == LogoTokenKind.KIND_INTEGER) {
                    receiver.right(token.value);
                    token = parser.getToken();
                } else {
                    notifyError("rightには角度を指定して下さい");
                    return;
                }
                break;

            case KIND_SETPS:
                token = parser.getToken();
                if (token.kind == LogoTokenKind.KIND_INTEGER) {
                    receiver.setPenSize((int)token.value);
                    token = parser.getToken();
                } else {
                    notifyError("setpsにはペンサイズを指定して下さい");
                    return;
                }
                break;

            case KIND_PENUP:
                receiver.penUp();
                token = parser.getToken();
                break;

            case KIND_PENDOWN:
                receiver.penDown();
                token = parser.getToken();
                break;

            case KIND_HIDETURTLE:
                receiver.hideTurtle();
                token = parser.getToken();
                break;

            case KIND_SHOWTURTLE:
                receiver.showTurtle();
                token = parser.getToken();
                break;

            case KIND_CLEARSCREEN:
                receiver.clearScreen();
                token = parser.getToken();
                break;

            case KIND_REPEAT:
                token = parser.getToken();
                if (token.kind != LogoTokenKind.KIND_INTEGER) {
                    notifyError("repeatには反復回数を指定して下さい");
                    return;
                }
                int count = (int)token.value;
                token = parser.getToken();
                if (token.kind != LogoTokenKind.KIND_BLOCK_BEGIN) {
                    notifyError("repeatにはブロック[]を指定して下さい");
                    return;
                }
                int blockStart = parser.getIndex();
                while (count-- > 0) {
                    parser.setIndex(blockStart);
                    invokeCommandImpl(level + 1);
                }
                token = parser.getToken();
                break;

            case KIND_TO:
                token = parser.getToken();
                if (token.kind != LogoTokenKind.KIND_IDENTIFIER) {
                    notifyError("TOには名前を指定して下さい");
                    return;
                }
                commandName = token.lexis;
                code = parser.getRestAsString();
                dictionary.put(commandName, code);
                token = parser.getToken();
                break;

            case KIND_IDENTIFIER:
                commandName = token.lexis;
                code = (String) dictionary.get(commandName);
                if (code == null) {
                    notifyError("定義されていないコマンドです：" + commandName);
                    return;
                }
                String savedCommand = parser.getSourceString();
                int savedPosition = parser.getIndex();
                parser.setSourceString(code);
                invokeCommandImpl(0);
                parser.setSourceString(savedCommand);
                parser.setIndex(savedPosition);
                token = parser.getToken();
                break;

            case KIND_SETPC:
                token = parser.getToken();
                Integer newPenColor = (Integer)colorTable.get(token.lexis);
                if (newPenColor == null) {
                    notifyError("未定義のペンの色です：" + token.lexis);
                    return;
                }
                receiver.setPenColor(newPenColor);
                token = parser.getToken();
                break;

            case KIND_BLOCK_BEGIN:
                token = parser.getToken();
                break;

            case KIND_BLOCK_END:
                if (level == 0) {
                    notifyError("ブロックの数が一致しません");
                    token = parser.getToken();
                    return;
                }
                return;

            default:
                notifyError("未定義のコマンドです");
                return;
            }
        }
    }
    void notifyError(String message) {
        throw new LogoCommandParseException(message);
    }
}
