package jp.asciimw.androidbook.chapter2.andlogo.core;

import java.util.HashMap;
import java.util.Map;

import jp.asciimw.androidbook.chapter2.andlogo.core.LogoToken.LogoTokenKind;



public class LogoCommandParser {
    private Map<String, LogoTokenKind> commandStringMap;
    private char[] str;
    private int length;
    private int index;

    public LogoCommandParser() {
        commandStringMap = new HashMap<String, LogoTokenKind>();
        commandStringMap.put("forward", LogoTokenKind.KIND_FORWARD);
        commandStringMap.put("fd", LogoTokenKind.KIND_FORWARD);
        commandStringMap.put("back", LogoTokenKind.KIND_BACK);
        commandStringMap.put("bk", LogoTokenKind.KIND_BACK);
        commandStringMap.put("right", LogoTokenKind.KIND_RIGHT);
        commandStringMap.put("rt", LogoTokenKind.KIND_RIGHT);
        commandStringMap.put("left", LogoTokenKind.KIND_LEFT);
        commandStringMap.put("lt", LogoTokenKind.KIND_LEFT);
        commandStringMap.put("penup", LogoTokenKind.KIND_PENUP);
        commandStringMap.put("pu", LogoTokenKind.KIND_PENUP);
        commandStringMap.put("pendown", LogoTokenKind.KIND_PENDOWN);
        commandStringMap.put("pd", LogoTokenKind.KIND_PENDOWN);
        commandStringMap.put("hideturtle", LogoTokenKind.KIND_HIDETURTLE);
        commandStringMap.put("ht", LogoTokenKind.KIND_HIDETURTLE);
        commandStringMap.put("showturtle", LogoTokenKind.KIND_SHOWTURTLE);
        commandStringMap.put("st", LogoTokenKind.KIND_SHOWTURTLE);
        commandStringMap.put("clearscreen", LogoTokenKind.KIND_CLEARSCREEN);
        commandStringMap.put("cs", LogoTokenKind.KIND_CLEARSCREEN);
        commandStringMap.put("repeat", LogoTokenKind.KIND_REPEAT);
        commandStringMap.put("rep", LogoTokenKind.KIND_REPEAT);
        commandStringMap.put("to", LogoTokenKind.KIND_TO);
        commandStringMap.put("setpc", LogoTokenKind.KIND_SETPC);
        commandStringMap.put("pc", LogoTokenKind.KIND_SETPC);
        commandStringMap.put("setps", LogoTokenKind.KIND_SETPS);
        commandStringMap.put("ps", LogoTokenKind.KIND_SETPS);
    }

    public int getIndex() {
        return index;
    }

    public void setIndex(int index) {
        if (this.index >= 0 && this.index <= length) {
            this.index = index;
            return;
        }
        this.index = length;
    }

    public void setSourceString(String newSourceString) {
        length = newSourceString.length();
        str = newSourceString.concat("\0").toCharArray();
        index = 0;
    }

    public String getSourceString() {
        return String.valueOf(str);
    }

    public String getRestAsString() {
        ignoreSpaces();
        String rest = new String(str, index, length - index + 1);
        index = length;
        return rest;
    }

    private void ignoreSpaces() {
        char c;
        do {
            c = str[index++];
        } while (c == ' ' || c == '\t');
        --index;
    }

    public LogoToken getToken() {
        LogoToken token = new LogoToken();
        StringBuffer lexis = new StringBuffer();
        char c;
        if (index >= length) {
            token.kind = LogoTokenKind.KIND_EOI;
            return token;
        }
        ignoreSpaces();
        c = str[index++];
        if (c == '[') {
            token.kind = LogoTokenKind.KIND_BLOCK_BEGIN;
            token.lexis = String.valueOf(c);
        } else if (c == ']') {
            token.kind = LogoTokenKind.KIND_BLOCK_END;
            token.lexis = String.valueOf(c);
        } else if (isAlphaChar(c)) {
            do {
                lexis.append(c);
                c = str[index++];
            } while (isAlphaChar(c));
            --index;
            token.lexis = lexis.toString();
            token.kind = LogoTokenKind.KIND_IDENTIFIER;
            LogoTokenKind keyword = (LogoTokenKind) commandStringMap.get(token.lexis);
            if (keyword != null) {
                token.kind = keyword;
            }
        } else if (isDigitChar(c)) {
            do {
                lexis.append(c);
                c = str[index++];
            } while (isDigitChar(c));
            boolean hasDecimalPart = false;
            if (c == '.') {
                do {
                    lexis.append(c);
                    c = str[index++];
                } while (isDigitChar(c));
                hasDecimalPart = true;
            }
            --index;
            token.lexis = lexis.toString();
            token.value = Float.valueOf(token.lexis).floatValue();
            if (hasDecimalPart) {
                token.kind = LogoTokenKind.KIND_FLOAT;
            } else {
                token.kind = LogoTokenKind.KIND_INTEGER;
            }
        } else if (c == 0) {
            --index;
            token.kind = LogoTokenKind.KIND_EOI;
        } else {
            --index;
            token.kind = LogoTokenKind.KIND_INVALID_TOKEN;
        }
        return token;
    }
    boolean isAlphaChar(char c) {
        return Character.isLetter(c);
    }
    boolean isDigitChar(char c) {
        return Character.isDigit(c);
    }
}
