package jp.asciimw.androidbook.chapter2.andlogo.gl;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;

import javax.microedition.khronos.opengles.GL10;

import android.graphics.Bitmap;
import android.graphics.Color;
import android.graphics.PointF;
import android.graphics.Rect;
import android.graphics.RectF;
import android.opengl.GLUtils;

public class GLTurtle {
    private int width;
    private int height;
    private Rect viewRect;
    private RectF rect;
    private float direction;
    private boolean visible;
    private boolean penUp;
    private int penWidth;
    private int penColor;
    private static final float[] UV = {
        0.0f, 0.0f, 0.0f, 1.0f, 
        1.0f, 0.0f, 1.0f, 1.0f, 
    };
    private static final float[] TURTLE_POS = {
        -0.5f,  0.5f, 0.0f,
        -0.5f, -0.5f, 0.0f,
         0.5f,  0.5f, 0.0f,
         0.5f, -0.5f, 0.0f,
    };

    public GLTurtle(Bitmap bitmap, GL10 gl10, int w, int h) {
        gl10.glEnable(GL10.GL_TEXTURE_2D);
        int[] buffers = new int[1];
        gl10.glGenTextures(1, buffers, 0);
        int texture = buffers[0];
        gl10.glBindTexture(GL10.GL_TEXTURE_2D, texture);
        gl10.glBlendFunc(GL10.GL_SRC_ALPHA, GL10.GL_ONE_MINUS_SRC_ALPHA);
        gl10.glEnable(GL10.GL_BLEND);
        GLUtils.texImage2D(GL10.GL_TEXTURE_2D, 0, bitmap, 0);
        gl10.glTexParameterf(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_MIN_FILTER, GL10.GL_NEAREST);
        gl10.glTexParameterf(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_MAG_FILTER, GL10.GL_NEAREST);
        bitmap.recycle();

        this.width = bitmap.getWidth();
        this.height = bitmap.getHeight();
        viewRect = new Rect(0, 0, w, h);
        init();
    }

    public void init() {
        this.rect = new RectF(0.0f, 0.0f, width, height);
        rect.offset(viewRect.right / 2 - width / 2, viewRect.bottom / 2 - height / 2);
        direction = 90.0f;
        visible = true;
        penUp = false;
        penColor = Color.RED;
        penWidth = 1;
    }

    public void draw(GL10 gl) {
        ByteBuffer byteBuffer = ByteBuffer.allocateDirect(UV.length * 4);
        byteBuffer.order(ByteOrder.nativeOrder());
        FloatBuffer floatBuffer = byteBuffer.asFloatBuffer();
        floatBuffer.put(UV);
        floatBuffer.position(0);
        gl.glEnableClientState(GL10.GL_TEXTURE_COORD_ARRAY);
        gl.glTexCoordPointer(2, GL10.GL_FLOAT, 0, floatBuffer);
        drawQuadrant(gl);
    }
    private void drawQuadrant(GL10 gl) {
        float w = (float) width / (float) viewRect.width() * 2;
        float h = (float) height / (float) viewRect.height() * 2;

        ByteBuffer byteBuffer = ByteBuffer.allocateDirect(TURTLE_POS.length * 4);
        byteBuffer.order(ByteOrder.nativeOrder());
        FloatBuffer floatBuffer = byteBuffer.asFloatBuffer();
        floatBuffer.put(TURTLE_POS);
        floatBuffer.position(0);
        gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
        gl.glVertexPointer(3, GL10.GL_FLOAT, 0, floatBuffer);
        gl.glMatrixMode(GL10.GL_MODELVIEW);
        gl.glLoadIdentity();
        gl.glPushMatrix();
        gl.glTranslatef(
                -1.0f + w / 2.0f + (rect.left / (float) viewRect.width()  * 2), 
                 1.0f - h / 2.0f - (rect.top  / (float) viewRect.height() * 2),
                 0.0f);
        gl.glScalef(w, h, 1.0f);
        gl.glRotatef(direction, 0, 0, 1.0f);
        gl.glDrawArrays(GL10.GL_TRIANGLE_STRIP, 0, 4);
        gl.glPopMatrix();
    }

    public PointF getCurrentPoint() {
        return new PointF(rect.centerX(), rect.centerY());
    }

    public int getCurrentPenColor() {
        return penColor;
    }

    public int getCurrentPenWidth() {
        return penWidth;
    }

    public void setPenColor(int color) {
        penColor = color;
    }

    public void setPenStrokeWidth(int width) {
        penWidth = width;
    }

    public float getX() {
        return rect.centerX();
    }

    public float getY() {
        return rect.centerY();
    }

    public void move(float dx, float dy) {
        rect.offset(dx, dy);
    }

    public void right(float angle) {
        left(-angle);
    }

    public void left(float angle) {
        direction += angle;
        while (direction > 360.0f) {
            direction -= 360.0f;
        }
        while (direction < 0.0f) {
            direction += 360.0f;
        }
    }

    public float getDirection() {
        return direction;
    }

    public void hide() {
        visible = false;
    }

    public void show() {
        visible = true;
    }

    public boolean isHidden() {
        return !visible;
    }

    public void penUp() {
        penUp = true;
    }

    public void penDown() {
        penUp = false;
    }

    public boolean isPenUp() {
        return penUp;
    }
}
