package jp.asciimw.androidbook.chapter2.andlogo.gl;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import jp.asciimw.androidbook.chapter2.andlogo.R;
import jp.asciimw.androidbook.chapter2.andlogo.core.LogoCommandReceiver;
import android.content.Context;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.PointF;
import android.opengl.GLSurfaceView.Renderer;
import android.util.TypedValue;

public class LogoRenderer implements Renderer, LogoCommandReceiver {

    class GLTurtleTrack {
        private int color;
        private int width;
        private float x1, y1, x2, y2;

        public GLTurtleTrack(float x1, float y1, float x2, float y2) {
            this.x1 = x1;
            this.y1 = y1;
            this.x2 = x2;
            this.y2 = y2;
        }
    }

    List<GLTurtleTrack> mLines;
    private GLTurtle mTurtle;
    private Context mContext;
    private int mWidth;
    private int mHeight;

    public LogoRenderer(Context context) {
        mLines = new CopyOnWriteArrayList<GLTurtleTrack>();
        mContext = context;
    }

    @Override
    public void onDrawFrame(GL10 gl) {
        gl.glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
        gl.glClear(GL10.GL_COLOR_BUFFER_BIT);
        gl.glDisable(GL10.GL_TEXTURE_2D);
        final int size = mLines.size();
        for (int i = 0; i < size; i++) {
            GLTurtleTrack targetTrack = mLines.get(i);
            drawLine(gl, targetTrack);
        }
        gl.glEnable(GL10.GL_TEXTURE_2D);
        gl.glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
        mTurtle.draw(gl);
    }

    private void drawLine(GL10 gl, GLTurtleTrack track) {
        float[] pos =
        { (track.x1 / mWidth) * 2.0f - 1.0f, -((track.y1 / mHeight) * 2.0f - 1.0f), 0.0f,
                (track.x2 / mWidth) * 2.0f - 1.0f, -((track.y2 / mHeight) * 2.0f - 1.0f), 0.0f };
        ByteBuffer byteBuffer = ByteBuffer.allocateDirect(pos.length * 4);
        byteBuffer.order(ByteOrder.nativeOrder());
        FloatBuffer floatBuffer = byteBuffer.asFloatBuffer();
        floatBuffer.put(pos);
        floatBuffer.position(0);
        gl.glColor4f(Color.red(track.color), Color.green(track.color), Color.blue(track.color), 1.0f);
        gl.glLineWidth(track.width);
        gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
        gl.glVertexPointer(3, GL10.GL_FLOAT, 0, floatBuffer);
        gl.glDrawArrays(GL10.GL_LINES, 0, 2);
    }

    @Override
    public void onSurfaceChanged(GL10 gl, int width, int height) {
        mWidth = width;
        mHeight = height;
        gl.glViewport(0, 0, width, height);
        mTurtle = new GLTurtle(BitmapFactory.decodeResource(mContext.getResources(), R.drawable.turtle), gl, width, height);
    }

    @Override
    public void onSurfaceCreated(GL10 gl, EGLConfig config) {

    }

    @Override
    public void showTurtle() {
        mTurtle.show();
    }

    @Override
    public void hideTurtle() {
        mTurtle.hide();
    }

    @Override
    public void setPenColor(int penColor) {
        mTurtle.setPenColor(penColor);
    }

    @Override
    public void setPenSize(int penSize) {
        mTurtle.setPenStrokeWidth(penSize);
    }

    @Override
    public void clearScreen() {
        mTurtle.init();
        mLines.clear();
    }

    @Override
    public void forward(float distance) {
        float pixelDistance = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, distance, mContext.getResources().getDisplayMetrics());
        float angle = (float) (mTurtle.getDirection() * Math.PI / 180.0f);
        float newX = (float) (pixelDistance * Math.cos(angle));
        float newY = (float) -(pixelDistance * Math.sin(angle));
        if (!mTurtle.isPenUp()) {
            addLine(newX, newY);
        }
        mTurtle.move(newX, newY);
    }

    void addLine(float x, float y) {
        mLines.add(getGLTurtleTrack(x, y));
    }

    private GLTurtleTrack getGLTurtleTrack(float x2, float y2) {
        PointF p = mTurtle.getCurrentPoint();
        GLTurtleTrack track = new GLTurtleTrack(p.x, p.y, p.x + x2, p.y + y2);
        track.color = mTurtle.getCurrentPenColor();
        track.width = mTurtle.getCurrentPenWidth();
        return track;
    }

    @Override
    public void back(float distance) {
        forward(-distance);
    }

    @Override
    public void right(float degree) {
        mTurtle.right(degree);
    }

    @Override
    public void left(float degree) {
        mTurtle.left(degree);
    }

    @Override
    public void penUp() {
        mTurtle.penUp();
    }

    @Override
    public void penDown() {
        mTurtle.penDown();
    }

}
