package jp.asciimw.androidbook.chapter2.weather;

import java.io.IOException;
import java.io.StringReader;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;

public class WeatherFeedParser {
    public Weather parse(String contents) throws XmlPullParserException, IOException {
        final XmlPullParserFactory factory = XmlPullParserFactory.newInstance();
        final XmlPullParser parser = factory.newPullParser();
        final StringReader reader = new StringReader(contents);
        try {
            parser.setInput(reader);
            return parseWeather(parser);
        } finally {
            reader.close();
        }
    }
    private Weather parseWeather(XmlPullParser parser) throws XmlPullParserException, IOException {
        Weather weather = new Weather();
        int type = -1;
        int depth = parser.getDepth();
        while (((type=parser.next()) != XmlPullParser.END_TAG || parser.getDepth() > depth) && type != XmlPullParser.END_DOCUMENT) {
            if (type == XmlPullParser.START_TAG) {
                final String name = parser.getName();
                if (name.equals("forecast_information")) {
                    weather.city = getCityName(parser);
                } else if(name.equals("current_conditions")) {
                    weather.weatherIconUrl = getIconUrl(parser);
                }
            }
        }
        return weather;
    }
    private String getCityName(XmlPullParser parser) throws XmlPullParserException, IOException {
        int type = -1;
        String name = "";
        String cityName = "";
        int depth = parser.getDepth();
        while (((type=parser.next()) != XmlPullParser.END_TAG || parser.getDepth() > depth) && type != XmlPullParser.END_DOCUMENT) {
            if (type == XmlPullParser.START_TAG) {
                name = parser.getName();
                if(name == null || name.equals("")) {
                    continue;
                }
                if(name.equals("city")) {
                    cityName = parser.getAttributeValue(null, "data");
                }
            } else if(type == XmlPullParser.END_TAG) {
                name = "";
            } 
        }
        return cityName;
    }
    private String getIconUrl(XmlPullParser parser) throws XmlPullParserException, IOException {
        int type = -1;
        String name = "";
        String iconUrl = "";
        int depth = parser.getDepth();
        while (((type=parser.next()) != XmlPullParser.END_TAG || parser.getDepth() > depth) && type != XmlPullParser.END_DOCUMENT) {
            if (type == XmlPullParser.START_TAG) {
                name = parser.getName();
                if(name == null || name.equals("")) {
                    continue;
                }
                if(name.equals("icon")) {
                    iconUrl = "http://www.google.com" + parser.getAttributeValue(null, "data");
                }
            } else if(type == XmlPullParser.END_TAG) {
                name = "";
            } 
        }
        return iconUrl;
    }
}
