package jp.asciimw.androidbook.chapter2.twitter;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;
import java.util.Map;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Bitmap.Config;
import android.graphics.BitmapFactory.Options;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.SimpleAdapter;

public class TweetAdapter extends SimpleAdapter {
    
    private List<Map<String, String>> data;

    public TweetAdapter(Context context, List<Map<String, String>> data) {
        super(  context, data, R.layout.list_item,
                new String[]{ "text", "user", "created_at" },
                new int[] {R.id._text, R.id._user, R.id._created_at}
               );
        this.data = data;
    }
    
    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        View view = super.getView(position, convertView, parent);
        ImageView icon = (ImageView)view.findViewById(R.id._icon);
        String iconUrl = data.get(position).get("icon");
        Bitmap b = ImageMap.getImage(iconUrl);
        
        if (b != null) {
            icon.setImageBitmap(b);
        } else {
            // Twitterからデータ取得(UIスレッド内で同期処理をしているので、UIが固まってしまう)
            b = loadImage(iconUrl);
            ImageMap.setImage(iconUrl, b);
            icon.setImageBitmap(b);
        }
       return view;
    }
    
    private Bitmap loadImage(String urlString) {
        try{
            URL url = new URL(urlString);
            Options options = new BitmapFactory.Options();
            options.inPreferredConfig = Config.RGB_565;
            Bitmap bmp = BitmapFactory.decodeStream(url.openStream(), null, options);
            return bmp;
        } catch (MalformedURLException e) {
            return null;
        } catch (IOException e) {
            return null;
        }
    }

}
