package jp.asciimw.androidbook.chapter2.twitter3;

import java.util.List;
import java.util.Map;

import android.content.Context;
import android.graphics.Bitmap;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.SimpleAdapter;
import android.widget.TextView;

public class TweetAdapter extends SimpleAdapter {
    
    private List<Map<String, String>> data;
    private LayoutInflater mInflater;

    private class ViewHolder {
        TextView user;
        TextView created_at;
        TextView text;
        ImageView icon;
    }
    
    public TweetAdapter(Context context, List<Map<String, String>> data) {
        super(  context, data, R.layout.list_item,
                new String[]{ "text", "user", "created_at" },
                new int[] {R.id._text, R.id._user, R.id._created_at}
               );
        this.data = data;
        mInflater = (LayoutInflater) context.
                getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    }
    
    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        ViewHolder holder;
        if (convertView == null) {
            convertView = mInflater.inflate(R.layout.list_item, null, false);
            holder = new ViewHolder();
            holder.user = (TextView) convertView.findViewById(R.id._user);
            holder.created_at = (TextView) convertView.findViewById(R.id._created_at);
            holder.text = (TextView) convertView.findViewById(R.id._text);
            holder.icon = (ImageView) convertView.findViewById(R.id._icon);
            convertView.setTag(holder);
        } else {
            holder = (ViewHolder) convertView.getTag();
        }
        Map<String,String> tweet = data.get(position);
        holder.user.setText(tweet.get("user"));
        holder.created_at.setText(tweet.get("created_at"));
        holder.text.setText(tweet.get("text"));
        String iconUrl = tweet.get("icon");
        holder.icon.setTag(iconUrl);

        Bitmap b = ImageMap.getImage(iconUrl);
        if (b != null) {
            holder.icon.setImageBitmap(b);
        } else {
            holder.icon.setImageDrawable(null);
            new SetImageTask(iconUrl, holder.icon).execute((Void) null);
        }
        return convertView;
    }
    
}
