package jp.asciimw.androidbook.chapter2.twitter3;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

public class TweetReader {

    private DefaultHttpClient mClient = new DefaultHttpClient();

    public List<Map<String,String>> getTweet(String keyword) {
        String url = String.format(
                "http://search.twitter.com/search.json?q=%s", keyword);
        return executeImpl(url);
    }
    
    private List<Map<String,String>> executeImpl(String url) {
        List<Map<String,String>> tweet = null;
        try {
            final HttpGet req = new HttpGet(url);
            final HttpResponse response = executeRequest(req);
            final int statusCode = response.getStatusLine().getStatusCode();
            final StringBuilder buf = new StringBuilder();
            final InputStream in = response.getEntity().getContent();
            final BufferedReader reader = new BufferedReader(new InputStreamReader(in));
            String l = null;
            while ((l = reader.readLine()) != null) {
                buf.append(l);
            }
            if (statusCode == 200) {
                tweet = parse(buf.toString());
            }
        } catch (IOException e) {
            e.printStackTrace();
        } catch (JSONException e) {
            e.printStackTrace();
        } catch (RuntimeException e) {
            e.printStackTrace();
        }
        return tweet;
    }
    
    private HttpResponse executeRequest(HttpRequestBase base) throws IOException {
        try {
            return mClient.execute(base);
        } catch (IOException e) {
            base.abort();
            throw e;
        }
    }
    
    private List<Map<String,String>> parse(String buff) throws JSONException {
        JSONTokener token = new JSONTokener(buff);
        JSONObject obj = new JSONObject(token);
        JSONArray results = obj.getJSONArray("results");
        List<Map<String,String>> tweetList = new ArrayList<Map<String,String>>();
        for(int i = 0; i < results.length(); ++i){
            JSONObject tw = results.getJSONObject(i);
            Map<String, String> tweet = new HashMap<String,String>();
            tweet.put("text", tw.get("text").toString());
            tweet.put("user", tw.get("from_user_name").toString());
            tweet.put("created_at", tw.get("created_at").toString());
            tweet.put("icon", tw.get("profile_image_url").toString());
            tweetList.add(tweet);
        }
        return tweetList;
    }
    
}
