package jp.asciimw.androidbook.chapter3.process.bar;

import android.app.Service;
import android.content.Intent;
import android.os.IBinder;
import android.os.RemoteException;
import android.util.Log;

public class BarService extends Service {

    private Bar mBar;
    private final IBarService.Stub mBinder = new IBarService.Stub() {

        @Override
        public Bar startBarActivity(Bar bar) throws RemoteException {
            return startActivity(bar);
        }

        @Override
        public void setBar(Bar bar) throws RemoteException {
            set(bar);
        }

        @Override
        public Bar getBar() throws RemoteException {
            return get();
        }

    };

    @Override
    public IBinder onBind(Intent intent) {
        Log.d(getString(R.string.app_name), "onBind: " + Thread.currentThread());
        mBar = intent.getParcelableExtra("bar");
        return mBinder;
    }

    private synchronized Bar startActivity(Bar bar) {
        Log.d(getString(R.string.app_name),
                "startActivity: " + Thread.currentThread());
        startActivity(new Intent(this, BarActivity.class).putExtra("bar", bar)
                .setFlags(
                        Intent.FLAG_ACTIVITY_NEW_TASK
                                | Intent.FLAG_ACTIVITY_SINGLE_TOP));
        return mBar;
    }

    private synchronized void set(Bar bar) {
        Log.d(getString(R.string.app_name), "set: " + Thread.currentThread());
        mBar = bar;
    }

    private synchronized Bar get() {
        Log.d(getString(R.string.app_name), "get: " + Thread.currentThread());
        return mBar;
    }

}
