package jp.asciimw.androidbook.chapter3.process.foo;

import jp.asciimw.androidbook.chapter3.process.bar.Bar;
import jp.asciimw.androidbook.chapter3.process.bar.IBarService;
import android.app.Activity;
import android.content.ComponentName;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.IBinder;
import android.os.RemoteException;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

public class FooActivity extends Activity {

    private Button mStart;
    private Button mSet;
    private EditText mSetIntValue;
    private EditText mSetStringValue;
    private EditText mSetIntentExtra;
    private Button mGet;
    private TextView mGetIntValue;
    private TextView mGetStringValue;
    private TextView mGetIntentExtra;
    private IBarService mService;
    private final ServiceConnection mConnection = new ServiceConnection() {

        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            mService = IBarService.Stub.asInterface(service);
        }

        @Override
        public void onServiceDisconnected(ComponentName name) {
            mService = null;
        }

    };

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        mStart = (Button) findViewById(R.id.start);
        mStart.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    if (mService != null) {
                        set(mService.startBarActivity(get()));
                    }
                } catch (RemoteException e) {
                }
            }
        });
        mSet = (Button) findViewById(R.id.set);
        mSet.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    if (mService != null) {
                        mService.setBar(get());
                    }
                } catch (RemoteException e) {
                }
            }
        });
        mSetIntValue = (EditText) findViewById(R.id.setIntValue);
        mSetStringValue = (EditText) findViewById(R.id.setStringValue);
        mSetIntentExtra = (EditText) findViewById(R.id.setIntentExtra);
        mGet = (Button) findViewById(R.id.get);
        mGet.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    if (mService != null) {
                        set(mService.getBar());
                    }
                } catch (RemoteException e) {
                }
            }
        });
        mGetIntValue = (TextView) findViewById(R.id.getIntValue);
        mGetStringValue = (TextView) findViewById(R.id.getStringValue);
        mGetIntentExtra = (TextView) findViewById(R.id.getIntentExtra);
        bindService(new Intent(IBarService.class.getName()).putExtra("bar",
                new Bar(0, "", new Intent().putExtra("intentExtra", ""))),
                mConnection, BIND_AUTO_CREATE);
    }

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        Log.d(getString(R.string.app_name),
                "onNewIntent: " + Thread.currentThread());
        Foo foo = intent.getParcelableExtra("foo");
        mGetIntValue.setText(String.valueOf(foo.getInt()));
        mGetStringValue.setText(foo.getString());
        mGetIntentExtra.setText(foo.getIntent().getStringExtra("intentExtra"));
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        unbindService(mConnection);
    }

    private Bar get() {
        int i = mSetIntValue.getText().length() != 0 ? Integer
                .valueOf(mSetIntValue.getText().toString()) : 0;
        String s = mSetStringValue.getText().toString();
        Intent intent = new Intent();
        intent.putExtra("intentExtra", mSetIntentExtra.getText().toString());
        return new Bar(i, s, intent);
    }

    private void set(Bar bar) {
        mGetIntValue.setText(String.valueOf(bar.getInt()));
        mGetStringValue.setText(bar.getString());
        mGetIntentExtra.setText(bar.getIntent().getStringExtra("intentExtra"));
    }

}