package jp.asciimw.androidbook.chapter3.trianglechecker.ndtest;


import org.openqa.selenium.By;

import junit.framework.TestCase;

import com.google.android.testing.nativedriver.client.AndroidNativeDriver;
import com.google.android.testing.nativedriver.client.AndroidNativeDriverBuilder;
import com.google.android.testing.nativedriver.client.AndroidNativeElement;

/**
 * TriangleActivityのテストケース実装(NativeDriver版)
 */
public class TriangleActivityTest extends TestCase {
    
    private static final String TESTEE
        = "jp.asciimw.androidbook.chapter3.trianglechecker.TriangleActivity";
    AndroidNativeDriver driver;
    
    @Override
    protected void setUp() throws Exception {
        driver = getDriver();
    }
    
    @Override
    protected void tearDown() throws Exception {
        driver.quit();
    }
    
    // 初期条件テスト
    public void testPreConditions() {
        driver.startActivity(TESTEE);
        assertEquals("10", driver.findElement(By.id("editText_a")).getText());
        assertEquals("10", driver.findElement(By.id("editText_b")).getText());
        assertEquals("10", driver.findElement(By.id("editText_c")).getText());
        assertEquals("", driver.findElement(By.id("error_message")).getText());
        assertEquals("", driver.findElement(By.id("result_message")).getText());
        assertNotNull(driver.findElement(By.id("do_check")));
    }
    
    public void test01() {
        driver.startActivity(TESTEE);
        inputText("editText_a", "2");
        inputText("editText_b", "3");
        inputText("editText_c", "4");
        assertTrue(driver.findElement(By.id("do_check")).isEnabled());
        buttonClick();
        assertEquals("", driver.findElement(By.id("error_message")).getText());
        assertEquals("不等辺三角形です",
                driver.findElement(By.id("result_message")).getText());
    }

    public void test04() {
        driver.startActivity(TESTEE);
        inputText("editText_a", "-1");
        inputText("editText_b", "2");
        inputText("editText_c", "2");
        assertFalse(driver.findElement(By.id("do_check")).isEnabled());
        assertEquals("", driver.findElement(By.id("result_message")).getText());
        assertEquals("３辺の長さは自然数で入力してください",
                driver.findElement(By.id("error_message")).getText());
    }
    
    public void test06() {
        driver.startActivity(TESTEE);
        inputText("editText_a", "a");
        inputText("editText_b", "2");
        inputText("editText_c", "2");
        assertFalse(driver.findElement(By.id("do_check")).isEnabled());
        assertEquals("", driver.findElement(By.id("result_message")).getText());
        assertEquals("３辺の長さは自然数で入力してください",
                driver.findElement(By.id("error_message")).getText());
    }

    private AndroidNativeDriver getDriver() throws Exception{
        return new AndroidNativeDriverBuilder().withDefaultServer().build();
    }
    
    // editTextに対して、keysを入力する
    private void inputText(String elementId, String keys) {
        AndroidNativeElement element = driver.findElement(By.id(elementId));
        element.clear();
        element.sendKeys(keys+"\n");
    }

    // 「チェックする」ボタンをクリックする
    private void buttonClick() {
        AndroidNativeElement element = driver.findElement(By.id("do_check"));
        element.click();
    }

}
