package jp.asciimw.androidbook.chapter3.trianglechecker.test;

import android.test.ActivityInstrumentationTestCase2;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import jp.asciimw.androidbook.chapter3.trianglechecker.TriangleActivity;
import jp.asciimw.androidbook.chapter3.trianglechecker.R;

/**
 * TriangleActivityのテストケース実装(Instrumentation版)
 */
public class TriangleActivityTest extends
        ActivityInstrumentationTestCase2<TriangleActivity> {
    
    private TriangleActivity activity;
    private EditText text_a;
    private EditText text_b;
    private EditText text_c;
    private TextView error;
    private TextView result;
    private Button button;
    
    public TriangleActivityTest() {
        super(TriangleActivity.class);
    }

    @Override
    protected void setUp() throws Exception {
        super.setUp();
        setActivityInitialTouchMode(false);
        activity = getActivity();
        text_a = (EditText)getActivity().findViewById(R.id.editText_a);
        text_b = (EditText)getActivity().findViewById(R.id.editText_b);
        text_c = (EditText)getActivity().findViewById(R.id.editText_c);
        error  = (TextView)getActivity().findViewById(R.id.error_message);
        result  = (TextView)getActivity().findViewById(R.id.result_message);
        button = (Button)getActivity().findViewById(R.id.do_check);
    }
    
    // 初期条件テスト
    public void testPreConditions() {
        assertNotNull(activity);
        assertEquals("10", text_a.getText().toString());
        assertEquals("10", text_b.getText().toString());
        assertEquals("10", text_c.getText().toString());
        assertEquals("", error.getText());
        assertEquals("", result.getText());
        assertNotNull(button);
    }
    
    public void test01() {
        inputText(text_a, "2");
        inputText(text_b, "3");
        inputText(text_c, "4");
        assertTrue(button.isEnabled());
        buttonClick();
        assertEquals(View.INVISIBLE, error.getVisibility());
        assertEquals(View.VISIBLE, result.getVisibility());
        assertEquals("不等辺三角形です", result.getText());
    }
    
    public void test05() {
        inputText(text_a, "1");
        inputText(text_b, "2");
        inputText(text_c, "3");
        assertFalse(button.isEnabled());
        assertEquals(View.INVISIBLE, result.getVisibility());
        assertEquals(View.VISIBLE, error.getVisibility());
        assertEquals("どの１辺も、他の２辺の和より長くはなりません", error.getText());
    }

    public void test06() {
        inputText(text_a, "a");
        inputText(text_b, "2");
        inputText(text_c, "2");
        assertFalse(button.isEnabled());
        assertEquals(View.INVISIBLE, result.getVisibility());
        assertEquals(View.VISIBLE, error.getVisibility());
        assertEquals("３辺の長さは自然数で入力してください", error.getText());
    }
    
    // editTextに対して、keysを入力する
    private void inputText(final EditText editText, String keys) {
        activity.runOnUiThread(new Runnable() {
            public void run() {
                editText.requestFocus();
            }
        });
        getInstrumentation().waitForIdleSync();
        sendKeys(KeyEvent.KEYCODE_DPAD_RIGHT);
        sendKeys(KeyEvent.KEYCODE_DPAD_RIGHT);
        sendKeys(KeyEvent.KEYCODE_DEL);
        sendKeys(KeyEvent.KEYCODE_DEL);
        sendKeys(keys);
        sendKeys(KeyEvent.KEYCODE_ENTER);
    }
    
    // 「チェックする」ボタンをクリックする
    private void buttonClick() {
        activity.runOnUiThread(new Runnable() {
            public void run() {
                button.performClick();
            }
        });
        getInstrumentation().waitForIdleSync();
    }

}
