package jp.asciimw.androidbook.chapter3.trianglechecker.model;

/** 三角形 */
public class Triangle {
    
    // 三角形の三辺の長さ
    private int a;
    private int b;
    private int c;
    
    /** 三角形の種類 */
    public enum Type {
        SCALENCE( 1, "不等辺三角形"),
        ISOSCELES( 2, "二等辺三角形"),
        REGULAR( 3, "正三角形");
        public final int id;
        public final String name;
        private Type(int id, String name){
            this.id = id;
            this.name = name;
        }
    }
    
    public Triangle(int a, int b, int c) {
        if (a <= 0 || b<= 0 || c<= 0) {
            throw new IllegalArgumentException("３辺の長さは自然数で入力してください");
        }
        if (a+b <= c || b+c <= a || c+a <= b) {
            throw new IllegalArgumentException("どの１辺も、他の２辺の和より長くはなりません");
        }
        this.a = a;
        this.b = b;
        this.c = c;
    }
    
    /** 三角形の種類を判定し、Triangle.Type で結果を返す */
    public Type checkType() {
        if ( (a == b) && (b == c) ) {
            return Triangle.Type.REGULAR;
        }
        if ( (a == b) || (b == c) || (c == a) ) {
            return Triangle.Type.ISOSCELES;
        }
        return Triangle.Type.SCALENCE;
    }

}