package jp.asciimw.androidbook.chapter3.particularservice;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.security.KeyFactory;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.PublicKey;
import java.security.cert.Certificate;
import java.security.cert.CertificateEncodingException;
import java.security.spec.X509EncodedKeySpec;
import java.util.ArrayList;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import javax.crypto.Cipher;

import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;

import android.app.ActivityManager;
import android.app.ActivityManager.RunningAppProcessInfo;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.os.AsyncTask;
import android.os.Handler;
import android.os.IBinder;
import android.os.Process;
import android.os.RemoteException;
import android.util.Base64;
import android.util.Log;
import android.widget.Toast;

public class ParticularService extends Service {
    private static final String PUBLIC_KEY =
            "MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQC6Way42GKWQpd+px+lgxfWtBYL"
          + "asKffb76uLRFrelcEGrq5G6vG22TVkdvJ1pCl4+ToYzqFS0FQQ+1IcHKRehqhbL3"
          + "VD2tvk1eWGObmNuizxVsoZZBENsCXurTuzsNBhLAZ3UTncLDBWuQ5ogWLLjim8nW"
          + "XCZAUPfJZK8TRok37QIDAQAB";
    private static final String TARGET_PACKAGE_NAME = "jp.asciimw.androidbook.chapter3.particularservice";
    private static final String URL = "http://192.168.11.2:8887/getname?";
    Handler mHandler;

    @Override
    public void onCreate() {
        super.onCreate();
        mHandler = new Handler();
    }

    private final IParticularService.Stub stub = new IParticularService.Stub() {

        @Override
        public void execute1() throws RemoteException {
            int callingPid = getCallingPid();
            int callingUid = getCallingUid();
            int myPid = Process.myPid();
            int myUid = Process.myUid();
            String callingProcessName = resolveProcessNameByProcessId(callingPid);
            showToast("MY PID:" + myPid + ", MY UID:" + myUid + ", CALLING PID:" + callingPid + ", CALLING UID:" + callingUid + ", CALLING PNAME:" + callingProcessName);

        }

        @Override
        public void execute2() throws RemoteException {
            int pid = resolvePidByProcessName(TARGET_PACKAGE_NAME);
            if (getCallingPid() == pid) {
                showToast("same pid");
            } else {
                showToast("different pid");
            }
        }

        @Override
        public void execute3() throws RemoteException {
            PackageManager packageManager = getPackageManager();
            String installerPackageName = packageManager.getInstallerPackageName(TARGET_PACKAGE_NAME);
            Log.v("Particular", "Installer package name:" + installerPackageName);
            if ("com.android.vending".equals(installerPackageName)
                    || "com.google.android.feedback".equals(installerPackageName)) {
                showToast(TARGET_PACKAGE_NAME + " was installed from Google Play Store.");
            } else {
                showToast(TARGET_PACKAGE_NAME + " was installed from somewhere other than Google Play Store.");
            }
        }

        @Override
        public void execute4() throws RemoteException {
            try {
                PackageInfo packageInfo = getPackageManager().getPackageInfo(TARGET_PACKAGE_NAME, PackageManager.GET_SIGNATURES);
                String fingerPrint = getFingerprint(packageInfo.applicationInfo);
                showToast("FINGER PRINT:" + fingerPrint);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        private String getFingerprint(ApplicationInfo info) throws IOException, NoSuchAlgorithmException, CertificateEncodingException {
            JarFile jar = new JarFile(new File(info.sourceDir));
            final JarEntry manifest = (JarEntry) jar.getEntry("AndroidManifest.xml");
            if (manifest == null) {
                return null;
            }
            BufferedReader br = new BufferedReader(new InputStreamReader(jar.getInputStream(manifest)));
            while (br.readLine() != null)
                ;
            br.close();
            final Certificate[] certificates = manifest.getCertificates();
            if (certificates == null) {
                return null;
            }
            MessageDigest md = MessageDigest.getInstance("MD5");
            md.reset();
            md.update(certificates[0].getEncoded());
            StringBuilder sb = new StringBuilder();
            for (byte b : md.digest()) {
                sb.append(Integer.toString((b & 0xff) + 0x100, 16).substring(1).toUpperCase());
                sb.append(':');
            }
            return sb.substring(0, sb.length() - 1);
        }

        private int resolvePidByProcessName(String name) {
            ActivityManager activityManager = (ActivityManager) getSystemService(Context.ACTIVITY_SERVICE);
            List<RunningAppProcessInfo> infoList = activityManager.getRunningAppProcesses();
            for (RunningAppProcessInfo info : infoList) {
                if (name.equals(info.processName)) {
                    return info.pid;
                }
            }
            return -1;
        }

        private String resolveProcessNameByProcessId(int id) {
            ActivityManager activityManager = (ActivityManager) getSystemService(Context.ACTIVITY_SERVICE);
            List<RunningAppProcessInfo> infoList = activityManager.getRunningAppProcesses();
            for (RunningAppProcessInfo info : infoList) {
                if (id == info.pid) {
                    return info.processName;
                }
            }
            return null;
        }

        @Override
        public void execute5() throws RemoteException {
            if (PackageManager.PERMISSION_GRANTED != checkCallingPermission("jp.asciimw.androidbook.chapter3.particularservice.permission.GETNAME")) {
                showToast("Permission is not granted.");
                //
                return;
            }
            try {
                final String fingerPrint = getFingerprint(getApplicationInfo());
                final String enc = encrypt(getApplicationContext(), fingerPrint);
                AsyncTask<Void, Void, String> task = new AsyncTask<Void, Void, String>() {

                    @Override
                    protected String doInBackground(Void... params) {
                        return request(getApplicationContext(), enc);
                    }

                    @Override
                    protected void onPostExecute(String result) {

                        showToast(result);
                    }
                };
                task.execute();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        private String encrypt(Context context, String org) {
            String result = null;
            try {
                PublicKey publicKey = KeyFactory.getInstance("RSA").generatePublic(new X509EncodedKeySpec(Base64.decode(PUBLIC_KEY, Base64.DEFAULT)));
                final Cipher cipher = Cipher.getInstance("RSA/NONE/PKCS1PADDING");
                cipher.init(Cipher.ENCRYPT_MODE, publicKey);
                byte[] b64 = Base64.encode(cipher.doFinal(org.getBytes()), Base64.DEFAULT);
                result = new String(b64);
            } catch (Exception e) {
                e.printStackTrace();
            }
            return result;
        }

        private String request(Context context, String enc) {
            try {
                List<NameValuePair> params = new ArrayList<NameValuePair>();
                params.add(new BasicNameValuePair("p", enc));
                String parameter = URLEncodedUtils.format(params, "UTF-8");
                HttpGet request = new HttpGet(URL + parameter);
                DefaultHttpClient client = new DefaultHttpClient();
                HttpResponse response = client.execute(request);
                return response.getStatusLine().getStatusCode() + " - " + EntityUtils.toString(response.getEntity());
            }
            catch (Exception e) {
                return "Error:" + e.getMessage();
            }
        }
    };

    private void showToast(final String message) {
        mHandler.post(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(ParticularService.this, message, Toast.LENGTH_LONG).show();
            }
        });
    }

    public IBinder onBind(Intent intent) {
        return stub;
    }

    @Override
    public boolean onUnbind(Intent intent) {
        return super.onUnbind(intent);
    }

}
