-module(getname).
-export([start/1, stop/0]).

start(Port) ->
  Dic = [ {"B0:F9:EF:62:CE:EE:9A:02:93:4C:18:EC:F8:89:FB:7C", "EGAWA Takashi"} ,
          {"XX:F9:EF:62:CE:EE:9A:02:93:4C:18:EC:F8:89:FB:7A", "Developer 1"} ,
          {"YY:FA:EF:62:CE:EE:9A:02:93:4C:18:EC:F8:89:FB:7A", "Developer 2"} ],
  mochiweb_http:start([{port, Port}, {loop, fun(Req) -> dispatch(Req, Dic) end}]).

stop() ->
  mochiweb_http:stop().

dispatch(Req, Dic) ->
  try
    [Action|_] = string:tokens(Req:get(path), "/"),
    Param = proplists:get_value("p", Req:parse_qs()),
    handle(Req, Action, Param, Dic)
  catch _:_ ->
    fail_auth(Req)
  end.

handle(Req, "getname", undefined, _) ->
  not_found(Req);

handle(Req, "getname", Param, Dic) ->
  Key = decript_req(Param),
  Req:ok({"text/html", hd([Y || {X,Y} <- Dic, X == Key])});
  
handle(Req, _, _, _) ->
  not_found(Req).

not_found(Req) ->
  Req:not_found().

fail_auth(Req) ->
  Req:respond({403, [{"Content-Type", "text/plain"}], <<"Forbidden.">>}).

decript_req(Val) ->
  Dec = base64:decode(Val),
  {ok, RSAPriPem} = file:read_file("private.pem"), 
  PemEntries = public_key:pem_decode(RSAPriPem), 
  RSAPriKey = public_key:pem_entry_decode(hd(PemEntries)), 
  binary_to_list(public_key:decrypt_private(Dec, RSAPriKey)).
